@@ -30,7 +30,7 @@
 #include <safer.h>
 #include <cast.h>
 #include <tean.h>
-#include <twofish/aes.h>
+#include <twofish.h>
 
 #define MAX_THREADS 100 /* After that pointers are being reused */
 
@@ -47,8 +47,7 @@ static int cur_thread=-1; /* No threads */
 static struct cast_key* ckey=NULL; /* For cast */
 static blf_ctx *c=NULL; /* For Blowfish */
 
-static cipherInstance *CI=NULL; /* For Twofish */
-static keyInstance *KI=NULL;    /* For Twofish */
+static TWOFISH_context *CT=NULL; /* For Twofish */
 
 static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
@@ -123,7 +122,7 @@ int get_key_size(int algorithm) /* In bytes */
       break;
 
    case TWOFISH:
-      return 32;
+      return 16;
       break;
 
    case ThreeWAY:
@@ -148,17 +147,15 @@ int init_mcrypt(const int algorithm, void * key, int lenofkey)
 
 char *keyword1, *keyword2, *keyword3; /* For 3DES and safer*/
 unsigned char *skey=NULL;
-char twokey[68];
-int i,j;
+word8 twokey[16]; /* 128 bits */
 
   saferkey=realloc(saferkey, (cur_thread+1)*sizeof(char*));
   kn1=realloc(kn1, (cur_thread+1)*sizeof(char*));
   kn2=realloc(kn2, (cur_thread+1)*sizeof(char*));
   kn3=realloc(kn3, (cur_thread+1)*sizeof(char*));
   c=realloc(c, (cur_thread+1) * sizeof(blf_ctx));
-  CI=realloc(CI, (cur_thread+1) * sizeof(cipherInstance));  
-  KI=realloc(KI, (cur_thread+1) * sizeof(keyInstance));    
-  ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));  
+  CT=realloc(CT, (cur_thread+1) * sizeof(TWOFISH_context));
+  ckey=realloc(ckey, (cur_thread+1) * sizeof(struct cast_key));
   keyword_given=realloc(keyword_given, (cur_thread+1)*sizeof(char*));
   algorithm_given=realloc(algorithm_given, (cur_thread+1)*sizeof(int));
 
@@ -246,20 +243,9 @@ int i,j;
 
    case TWOFISH:
 
-      Bzero( &KI[cur_thread], sizeof(keyInstance));
-      Bzero( &CI[cur_thread], sizeof(cipherInstance));      
       Bzero( twokey, sizeof(twokey));
-      
-      cipherInit( &CI[cur_thread], MODE_ECB, NULL);
-
-      j=0;
-      for (i=0;i<lenofkey;i++) {
-        sprintf( &twokey[j], "%.2x", (unsigned char) keyword_given[cur_thread][i]);
-        j+=2;
-      }
-      
-        j=makeKey ( &KI[cur_thread], DIR_ENCRYPT, lenofkey*8,  twokey);
-        if (j<0) { return (-1); } 
+      memmove( twokey, keyword_given[cur_thread], 16); /* 128 bit only */
+      twofish_setkey ( &CT[cur_thread], twokey, 16);
 
       return cur_thread;
       break;
@@ -320,7 +306,7 @@ int mcrypt(int thread, void *plaintext)
       break;
 
    case TWOFISH:
-      blockEncrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext); /* 16 bytes */ 
+      twofish_encrypt(&CT[thread], plaintext, plaintext); /* 16 bytes */ 
       return 0;
       break;
 
@@ -380,7 +366,7 @@ int mdecrypt(int thread, void *plaintext)
       break;
 
    case TWOFISH:
-      blockDecrypt(&CI[thread], &KI[thread], plaintext, 16*8, plaintext); /* 16 bytes */
+      twofish_decrypt(&CT[thread], plaintext, plaintext); /* 16 bytes */
       return 0;
       break;
 
