@@ -8,6 +8,7 @@
 # define LIBDEFS_H
 # include <libdefs.h>
 #endif
+#include <swap.h>
 
 #define ROUNDS 32
 #define DELTA 0x9e3779b9 /* sqr(5)-1 * 2^31 */
@@ -19,10 +20,16 @@
  **********************************************************/
 
 void tean(word32 *k, word32 *v, long N) {
+#ifdef WORDS_BIGENDIAN
   word32 y=v[0], z=v[1];
+#else
+  word32 y=byteswap(v[0]), z=byteswap(v[1]);
+#endif  
   word32 limit,sum=0;
+
   if(N>0) { /* ENCRYPT */
     limit=DELTA*N;
+#ifdef WORDS_BIGENDIAN
     while(sum!=limit) {
       y+=((z<<4)^(z>>5)) + (z^sum) + k[sum&3];
       sum+=DELTA;
@@ -34,9 +41,26 @@ void tean(word32 *k, word32 *v, long N) {
       z-=((y<<4)^(y>>5)) + (y^sum) + k[(sum>>11)&3];
       sum-=DELTA;
       y-=((z<<4)^(z>>5)) + (z^sum) + k[sum&3];
+#else
+    while(sum!=limit) {
+      y+=((z<<4)^(z>>5)) + (z^sum) + byteswap(k[sum&3]);
+      sum+=DELTA;
+      z+=((y<<4)^(y>>5)) + (y^sum) + byteswap(k[(sum>>11)&3]);
+    }
+  } else { /* DECRYPT */
+    sum=DELTA*(-N);
+    while(sum) {
+      z-=((y<<4)^(y>>5)) + (y^sum) + byteswap(k[(sum>>11)&3]);
+      sum-=DELTA;
+      y-=((z<<4)^(z>>5)) + (z^sum) + byteswap(k[sum&3]);
+#endif
     }
   }
+#ifdef WORDS_BIGENDIAN
   v[0]=y; v[1]=z;
+#else
+  v[0]=byteswap(y); v[1]=byteswap(z);
+#endif  
 }
 
 void cl_enc_block(word32 *k, word32 *v) {
