@@ -36,6 +36,10 @@
 # include <sys/types.h>
 #endif
 
+#ifdef HAVE_PWD_H
+# include <pwd.h>
+#endif
+
 #ifdef HAVE_FCNTL_H
 # include <fcntl.h>
 #endif
