@@ -1,3 +1,19 @@
+/*
+ *    This program is free software; you can redistribute it and/or modify
+ *    it under the terms of the GNU General Public License as published by
+ *    the Free Software Foundation; either version 2 of the License, or
+ *    (at your option) any later version.
+ *
+ *    This program is distributed in the hope that it will be useful,
+ *    but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *    GNU General Public License for more details.
+ *
+ *    You should have received a copy of the GNU General Public License
+ *    along with this program; if not, write to the Free Software
+ *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+*/
+
 #ifndef DEFINES_H
 # define DEFINES_H
 # include <defines.h>
@@ -13,8 +29,15 @@ void* fixkey(char * key, unsigned int* len, int bit7_flag, int hexflag,int algor
 char *tmp;
 char* keyword;
 char *tmpkey;
-int keysize=get_key_size(algorithm);
+int keysize;
 
+  if (hexflag==TRUE) {
+   keysize=2*get_key_size(algorithm);
+  }
+  else {
+   keysize=get_key_size(algorithm);
+  }
+ 
   if (len!=0) {
     tmpkey=malloc(*len);
   }
@@ -22,7 +45,7 @@ int keysize=get_key_size(algorithm);
     tmpkey=malloc(128);
   }
 
-keyword=calloc(1, get_key_size(algorithm));
+  keyword=calloc(1, get_key_size(algorithm));
 
 
   if (key==NULL) 	{
