@@ -34,40 +34,46 @@ struct flock* file_lock(short type, short whence)
   return &ret ;
 }
                                                                               
-void read_lock(int fd)   /* a shared lock on an entire file */
+int read_lock(int fd)   /* a shared lock on an entire file */
 {
+int x;
 
   Sigfunc *old=Signal(SIGALRM, time_out);
   alarm(LOCK_TIMEOUT);
 
-  fcntl(fd, F_SETLKW, file_lock(F_RDLCK, SEEK_SET));
+  x=fcntl(fd, F_SETLKW, file_lock(F_RDLCK, SEEK_SET));
 
   alarm(0);
   Signal(SIGALRM, old);
-
+  return x;
 }
                         
-void write_lock(int fd)  /* an exclusive lock on an entire file */
+int write_lock(int fd)  /* an exclusive lock on an entire file */
 {
+int x;
+
   Sigfunc* old=Signal(SIGALRM, time_out);
   alarm(LOCK_TIMEOUT);
 
-  fcntl(fd, F_SETLKW, file_lock(F_WRLCK, SEEK_SET));
+  x=fcntl(fd, F_SETLKW, file_lock(F_WRLCK, SEEK_SET));
   alarm (0);
   Signal(SIGALRM, old);  
+  return x;
 }
 
-void append_lock(int fd) 
+int append_lock(int fd) 
  /* a lock on the _end_ of a file -- other
   * processes may access existing records 
   */
 {
+int x;
   Sigfunc* old=Signal(SIGALRM, time_out);
   alarm(LOCK_TIMEOUT);
 
-  fcntl(fd, F_SETLKW, file_lock(F_WRLCK, SEEK_END));
+  x=fcntl(fd, F_SETLKW, file_lock(F_WRLCK, SEEK_END));
   alarm(0);
   Signal(SIGALRM, old);
+  return x;
 }
 
 void unlock(int fd)  /* unlock */
@@ -79,13 +85,13 @@ void unlock(int fd)  /* unlock */
 void file_lock(short type, short whence)
 {}
                                                                               
-void read_lock(int fd) 
+int read_lock(int fd) 
 {}
                         
-void write_lock(int fd)  
+int write_lock(int fd)  
 {}
 
-void append_lock(int fd) 
+int append_lock(int fd) 
 {}
 
 void unlock(int fd)  /* unlock */
@@ -97,5 +103,5 @@ void unlock(int fd)  /* unlock */
 void time_out(int signo) {
 
 fprintf(stderr, _("Timeout while waiting for file lock.\n"));
-exit(-1);
+ /*exit(-1);*/
 }
