@@ -36,38 +36,43 @@
 #include <hex.h>
 #include <keys.h>
 #include <random.h>
+#include <gaa.h>
+#include <errors.h>
 
- unsigned int bit7_flag=-1, hexflag=FALSE;
+ unsigned int bit7_flag, hexflag=FALSE;
  unsigned int stream_flag=FALSE;
  unsigned int mode=CBC; /* 0:cbc 1:ecb */
  unsigned int type=BLOWFISH; /* default: blowfish */
  int quiet=TRUE; /* silent by default */
  int unlink_flag=FALSE;
+ int sunlink_flag=FALSE; 
  int bare_flag=FALSE, real_random_flag=FALSE;
  char *outfile=0;
 
-
 void usage(char *name)
 {
 
-     fprintf(stderr,_("Usage: mcrypt [-dLFu87bhvrx] [-f keyfile] [-k keyword] [-m mode] [-a algorithm] [file ...]\n\n"));
+     fprintf(stderr,_("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k keyword] [-m mode] [-a algorithm] [-c config_file] [file ...]\n\n"));
 
      fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
      fprintf(stderr,_("Legal parameters are:\n\
 \n\
  -d --decrypt                 decrypts\n\
- -a --algorithm=ALGORITHM     specify the encryption and decryption algorithm.\n\
+ -c --config FILENAME         Use the configuration file FILENAME.\n\
+                              If not specified the .mcryptrc in your home\n\
+                              directory is used.\n\
+ -a --algorithm ALGORITHM     specify the encryption and decryption algorithm.\n\
                               Legal parameters are: 3way, des, 3des, gost, tea,\n\
                               safer64, safer128, cast128, crypt, twofish\n\
                               and blowfish. (Default is blowfish)\n\
- -m --mode=MODE               specify the mode (cfb, ecb or cbc).\n\
+ -m --mode MODE               specify the mode (cfb, ecb or cbc).\n\
                               (Default is cbc)\n\
- -k --key=KEYWORD             keyword to use.\n\
- -f --keyfile=FILENAME        file to read the keyword from.\n\
+ -k --key KEY1, KEY2, ...     keyword(s) to use.\n\
+ -f --keyfile FILENAME        file to read the keyword from.\n\
  -x --hex                     keyword in hex.\n\
- -7 --7bit                    keyword in 7bit mode. (default)\n\
- -8 --8bit                    keyword in 8bit mode.\n\
- -u --unlink                  unlink (and destroy) the input file after\n\
+ -i --bit MODE                keyword bit mode. Can be 7 or 8bit.\n\
+ -u --unlink                  unlink the input file after encryption/decryption\n\
+ -s --sunlink                 unlink and destroy the input file after\n\
                               encryption/decryption\n\
  -r --random                  Use real random data (if your system supports it).\n\
  -L --license                 displays license information.\n\
@@ -76,46 +81,46 @@ void usage(char *name)
  -b --bare                    Do not keep algorithm information in the encrypted\n\
                               file.\n\
  -F --force                   forces output to stdout.\n\
- \n\
+\n\
  -h --help                    prints this help\n\
  -v --version                 prints the version number\n\
- \n\
+\n\
 Report bugs to nmav@i-net.paiko.gr.\n\n"));
 }
 
+void mcrypt_license() {
+  fprintf (stderr,_("\nCopyright (C) 1998,1999 Nikos Mavroyanopoulos\n\
+This program is free software; you can redistribute it and/or modify \n\
+it under the terms of the GNU General Public License as published by \n\
+the Free Software Foundation; either version 2 of the License, or \n\
+(at your option) any later version. \n\
+\n\
+This program is distributed in the hope that it will be useful, \n\
+but WITHOUT ANY WARRANTY; without even the implied warranty of \n\
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n\
+GNU General Public License for more details. \n\
+\n\
+You should have received a copy of the GNU General Public License \n\
+along with this program; if not, write to the Free Software \n\
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
+}
 
+void mcrypt_version() {
 
+  fprintf (stderr, _("Mini-crypt v.%s\n"),VERSION);
+  fprintf (stderr, _("Copyright (C) 1998,1999 Nikos Mavroyanopoulos (nmav@i-net.paiko.gr)\n"));
+
+}
 
 /*				The main    */
 int main(int argc,char **argv)
 {
-	short int ch,ein=0,din=0,kin=0,force=0,return_val=0;
+        char tmperr[128];
+	short int ein=0,din=0,kin=0,force=0,return_val=0;
 	char *einfile=0 ,*dinfile=0, *keyword=0,*modet=0,*typet=0;
-	char **file, **keyfile=NULL;
-	extern char *optarg;
-	int x,y,i,file_count=0, keys=0;
-
-
-struct option longopts[] = {
-	    { "version", 0, 0, 'v' },
-	    { "help", 0, 0, 'h' },
-	    { "bare", 0, 0, 'b' },	    
-	    { "decrypt", 0, 0, 'd' },
-	    { "mode", 1, 0, 'm' },
-	    { "algorithm", 1, 0, 'a' },
-	    { "key", 1, 0, 'k' },
-	    { "hex", 0, 0, 'x' },
-	    { "random", 0, 0, 'r' },	    
-	    { "keyfile", 1, 0, 'f' },
-	    { "force", 0, 0, 'F' },
-	    { "7bit", 0, 0, '7' },	    
-	    { "8bit", 0, 0, '8' },
-	    { "quiet", 0, 0, 'q'},
-	    { "verbose", 0, 0, 'V'},	    
-	    { "license", 0, 0, 'L'},
-	    { "unlink", 0, 0, 'u'},	    
-	    { 0, 0, 0, 0 }
-       	  };
+	char **file, **keyfile=NULL, *cfile;
+	int x,y,i,file_count=0, keys=0, used_algo=FALSE, v;
+        gaainfo info;
 
 #ifdef HAVE_SIGNAL_H
  Signal( SIGINT , shandler);
@@ -135,10 +140,6 @@ struct option longopts[] = {
   textdomain (PACKAGE);
 #endif
 
-/* For RAND32... Called only here */
-#ifndef HAVE_DEV_RANDOM
- srand(time(0));
-#endif
 
 /* Examine how we were called */
    if (strstr(argv[0],"decrypt")!=NULL) {
@@ -152,127 +153,105 @@ struct option longopts[] = {
      ein=TRUE; /* It will change by the arguments */
    }
 
+        if((v = gaa(argc, argv, &info)) != -1) {
+          err_quit(_("Error in arguments.\n"));
+        }
+
+
+          if (info.config_file!=NULL) {
+            cfile=malloc(strlen(info.config_file));
+            strcpy(cfile,info.config_file);
+            if (check_file(cfile)!=1) {
+              err_warn ("Warning: Cannot access config file.\n");
+            }
+          }
+          else {
+#ifdef HAVE_GETPWUID          
+            cfile=get_cfile(getuid(), ".mcryptrc");
+#else
+            cfile=malloc(11);
+            strcpy(cfile,".mcryptrc");
+#endif          
+          }
+        
+        /* If config file exists */
+         if (check_file(cfile)==1) {
+          if((v = gaa_file(cfile, &info)) != -1) {
+               /* gets options from file 'config' */
+            err_quit(_("Error in the configuration file.\n"));
+          }
+         }
+         
+         free(cfile);
+
+        
+
+
+
+
 
             /* File pointers are as much as the file arguments */
 
-        file=malloc((argc-optind)*sizeof(char*)); 
+        file=malloc((info.size)*sizeof(char*)); 
+    
+        if (info.ed_specified!=0) {
+          din=info.din;
+          ein=info.ein;
+        }
 
-	while( (ch=getopt_long(argc,argv,"F78bLVxuqrda:m:k:f:hv",longopts, (int *)0 )) != EOF )
-
-		switch(ch) {
-	
-		case 'v':
-			printf (_("Mini-crypt v.%s\n"),VERSION);
-			printf (_("Copyright (C) 1998,1999 Nikos Mavroyanopoulos (nmav@i-net.paiko.gr)\n"));
-			return 0;
-			break;
-
-		case 'L':
-			printf (_("\nCopyright (C) 1998,1999 Nikos Mavroyanopoulos\n\
-This program is free software; you can redistribute it and/or modify \n\
-it under the terms of the GNU General Public License as published by \n\
-the Free Software Foundation; either version 2 of the License, or \n\
-(at your option) any later version. \n\
-\n\
-This program is distributed in the hope that it will be useful, \n\
-but WITHOUT ANY WARRANTY; without even the implied warranty of \n\
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the \n\
-GNU General Public License for more details. \n\
-\n\
-You should have received a copy of the GNU General Public License \n\
-along with this program; if not, write to the Free Software \n\
-Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
-			                                                
-			return 0;
-			break;			
-
-		case 'd':
-			din=TRUE;
-			ein=FALSE;
-			break;
-
-		case 'r':
+        if (info.real_random_flag==TRUE) {
 #ifdef HAVE_DEV_RANDOM
-			real_random_flag=TRUE;
-			break;
+          real_random_flag=TRUE;
 #else
-                        fprintf (stderr, _("This system does not support real random data.\n"));
-                        exit(1);
+          err_warn (_("This system does not support real random data.\n"));
 #endif
+        }
 
-		case 'F':
-			force=TRUE;
-			break;
+		force=info.force;
+		bare_flag=info.bare_flag;
+                unlink_flag=info.unlink_flag;
+                sunlink_flag=info.sunlink_flag;
+		quiet=info.quiet;
+                hexflag=info.hexflag;
 
-		case 'b':
-			bare_flag=TRUE;
-			break;
+                if (info.bit==7) {
+                  if (hexflag==TRUE) 
+                    err_quit(_("Hex mode and 7bit mode are mutually exclusive\n"));
+                  bit7_flag=BITS7;
+                }
+                else {
+                  bit7_flag=BITS8; /* default */
+                }
+              
 
-		case 'u':
-			unlink_flag=TRUE;
-			break;			
+                if (info.keyfile!=NULL) {
+                  kin=2;
+                  keyfile = read_key_file(info.keyfile, &keys);
+                  if (keyfile==NULL) kin=0;
+                  if (keys==0) kin=0;
+               }
+               else {
+                if (info.keys!=NULL) {
+                 kin=2;
+                 keyfile=malloc(info.keysize*sizeof(char*));
+                 keys=info.keysize;
+                
+                 for (i=0; i<info.keysize; i++) {
+                   keyfile[i]=malloc(strlen(info.keys[i])+10);
+                   strcpy(keyfile[i],info.keys[i]);
+                 }
+                }
+               }
 
-		case 'q':
-			quiet=TRUE;
-			break;
-
-		case 'x':
-                        if (bit7_flag==BITS7) {
-                          fprintf (stderr, _("--7bit and --hex parameters are mutually exclusive.\n"));
-                          fprintf (stderr, _("Specify the --8bit parameter since 7bit is the default.\n"));
-                          exit(1);
-                         }
-                        bit7_flag=BITS8;
-			hexflag=TRUE;
-			break;
-
-		case 'V':
-			quiet=FALSE;
-			break;
-
-		case '7':
-                       if (hexflag==TRUE) {
-                         fprintf (stderr, _("--7bit and --hex parameters are mutually exclusive.\n"));
-                         exit(1);
-                        }
-			bit7_flag=BITS7;
-			break;			
-
-		case '8':
-			bit7_flag=BITS8;
-			break;			
-
-		case 'f':
-                        if (kin==1) {
-                          fprintf(stderr, _("--key and --keyfile parameters are mutually exclusive.\n"));
-                          exit(1);
-                        }
-			kin=2;
-			keyfile = read_key_file(optarg, &keys);
-			if (keys==0) kin=0;
-			break;
-
-		case 'k':
-                        if (kin==2) {
-                          fprintf(stderr, _("--key and --keyfile parameters are mutually exclusive.\n"));
-                          exit(1);
-                        }
-			kin=1;
-                        keyfile=malloc(1*sizeof(char*));
-                        keyfile[0]=malloc(strlen(optarg)+10);
-                        keyword=keyfile[0];
-			strcpy(keyword,optarg);                        
-			break;			
-
-		case 'm':
-			modet= malloc(strlen(optarg)+1);
-			strcpy(modet,optarg);
+              if (info.mode!=NULL) {
+		modet= malloc(strlen(info.mode)+1);
+		strcpy(modet,info.mode);
 			if (strncasecmp(modet,"ECB",3)==0) {
 			  mode=ECB;
 			}
-                        else {
-                          if (strncasecmp(modet,"CBC",3)==0) {
-                            mode=CBC;
+	                else {
+                         if (strncasecmp(modet,"CBC",3)==0) {
+                           mode=CBC;
 			  }
 			  else {
                             if (strncasecmp(modet,"CFB",3)==0) {
@@ -280,15 +259,21 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 	  	            }
 	  	            else {
                               mode=CBC; /* Default */
+
+                              sprintf(tmperr, _("No such mode: %s\n"),modet);
+                              err_warn (tmperr);
+                              err_warn (_("Assuming CBC.\n"));
 	  	            }
 			  }
                         
                         }
 			free(modet);
-			break;
-		case 'a':
-			typet= malloc(strlen(optarg)+1);
-			strcpy(typet,optarg);
+                    }
+                    
+                    if (info.algorithm!=NULL) {
+                        used_algo=TRUE;
+			typet= malloc(strlen(info.algorithm)+1);
+			strcpy(typet,info.algorithm);
 			if (strncasecmp(typet,"DES",3)==0) {
 			  type=DES;
 			 }
@@ -334,6 +319,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
                                           }
                                           else { /* Default */
                                            type=BLOWFISH;
+                                           sprintf(tmperr, _("No such algorithm: %s\n"),typet);
+                                           err_warn (tmperr);
+                                           err_warn (_("Assuming Blowfish.\n"));
                                           }
                                        }
                                     }
@@ -346,40 +334,34 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
                          }
                         }
 			free(typet);
-			break;
+                      } /* if */ 
 
 
-		case '?':
-		case 'h':
-		default:
-		     usage(argv[0]);
-		     return 1;
-		     break;
-
-
-		}
-
 #ifdef HAVE_UMASK
   umask(066);
 #endif
 
-/* 7 bits in the key */
-if (bit7_flag==-1) bit7_flag=BITS7; /* The default behaviour */
+/* For RAND32... Called only here */
+#ifndef HAVE_DEV_RANDOM
+ if (quiet<=TRUE) err_warn (stderr, "Warning: Using pseudo random numbers\n");
+ srand(time(0)*getpid());
+#endif
+
 
 /* Normal startup */
 
 
 /* ok now how many files were specified? */
     i=0;
-    file_count+= (argc - optind);
+    file_count+= (info.size);
 
 
     if (file_count==0) {
         stream_flag=TRUE;
     }  
 
-    while (optind < argc) {
-        file[i]=argv[optind++];
+    while (i<info.size) {
+        file[i]=info.input[i];
         i++;
     }
 
@@ -403,7 +385,8 @@ for(i=0;i<file_count;i++) {
 
 #ifdef HAVE_STAT
     if ( is_normal_file( file[i]) == FALSE && stream_flag==FALSE ) {
-      fprintf( stderr, _("%s: %s is not a regular file. Skipping...\n"), argv[0], file[i]);
+      sprintf(tmperr, _("%s: %s is not a regular file. Skipping...\n"), argv[0], file[i]);
+      err_warn (tmperr);
       outfile=NULL;
       continue; /* next */
     }
@@ -420,17 +403,17 @@ for(i=0;i<file_count;i++) {
 	if (din==TRUE) {
             if (stream_flag!=TRUE) dinfile=file[i];
             if ( (isatty(fileno( (FILE *)(stdin) ) ) == 1) && (stream_flag==TRUE) && (force==0) ) { /* not a tty */        
-              fprintf(stderr,_("%s: Encrypted data will not be read from a terminal.\n"),argv[0]);
-              fprintf(stderr,_("Redirect the input instead.\nUse the --help parameter for more help.\n"));
-               exit(-1);
+              sprintf(tmperr, _("%s: Encrypted data will not be read from a terminal.\n"),argv[0]);
+              err_warn (tmperr);
+              err_quit (_("Redirect the input instead.\nUse the --help parameter for more help.\n"));
             }
         } 
         else { /* called as mcrypt or something like that */
             if (stream_flag!=TRUE) einfile=file[i];
             if ( (isatty(fileno( (FILE *)(stdout) ) ) == 1) && (stream_flag==TRUE) && (force==0) ) { /* not a tty */        
-              fprintf(stderr,_("%s: Encrypted data will not be written to a terminal.\n"),argv[0]);
-              fprintf(stderr,_("Redirect the output instead.\nUse the --help parameter for more help.\n"));
-              exit(-1);
+              sprintf(tmperr, _("%s: Encrypted data will not be written to a terminal.\n"),argv[0]);
+              err_warn (tmperr);
+              err_quit(_("Redirect the output instead.\nUse the --help parameter for more help.\n"));
             }            
         }
 
@@ -448,7 +431,11 @@ for(i=0;i<file_count;i++) {
   	      strncat(outfile,dinfile,y-4);
 #ifdef HAVE_STAT
               /* But if it exists exit */
-              if (check_file(outfile)!=0) {fprintf(stderr,_("%s: file %s exists... skipping...\n"), argv[0], outfile); continue;}
+              if (check_file(outfile)!=0) {
+                sprintf(tmperr, _("%s: file %s exists... skipping...\n"), argv[0], outfile);
+                err_warn (tmperr);
+                continue;
+              }
 #endif  	      
             }
             else { /* append .dec */
@@ -462,12 +449,20 @@ for(i=0;i<file_count;i++) {
 	    outfile= calloc(strlen(einfile)+5,1);
 	    strcpy(outfile,einfile);
             /* if file has already the .enc ignore it */
-            if (strstr( outfile, ".enc")!=NULL) {fprintf(stderr,_("%s: file %s has the .enc suffix... skipping...\n"), argv[0], outfile); continue;}
+            if (strstr( outfile, ".enc")!=NULL) {
+              sprintf(tmperr, _("%s: file %s has the .enc suffix... skipping...\n"), argv[0], outfile);
+              err_warn (tmperr);
+              continue;
+            }
 
 	    strcat(outfile,".enc");
 #ifdef HAVE_STAT
               /* But if it exists exit */
-              if (check_file(outfile)!=0) {fprintf(stderr,_("%s: file %s exists... skipping...\n"), argv[0], outfile); continue;}
+              if (check_file(outfile)!=0) {
+                sprintf(tmperr, _("%s: file %s exists... skipping...\n"), argv[0], outfile);
+                err_warn (tmperr);
+                continue;
+              }
 #endif  	      
 
 	 }
@@ -489,9 +484,12 @@ for(i=0;i<file_count;i++) {
                  type=NO_TYPE; /* Lock failed */
                }
                else { /* Unix Crypt */
-                 fprintf(stderr,_("%s: no headers. Assuming encrypted file with Unix crypt.\n"),argv[0]); 
+                if (used_algo==FALSE) {
+                 sprintf(tmperr, _("%s: no headers. Assuming encrypted file with Unix crypt.\n"),argv[0]);
+                 err_warn (tmperr);
                  fflush(stderr);
                  type=CRYPT;
+                }
                }
              }
            }
@@ -521,7 +519,7 @@ for(i=0;i<file_count;i++) {
                                  x=1;
                                  break;
                                 default:
-	                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
+	                         err_warn(_("Algorithm unsupported in this version\n"));
         	                 x=1;
                 	         break;
 	                      }
@@ -530,7 +528,7 @@ for(i=0;i<file_count;i++) {
                           break;
                         default:
                           x=1;
-                          fprintf(stderr,_("Mode unsupported in this version\n"));
+                          err_warn(_("Mode unsupported in this version\n"));
                           break;
                       }
                       
@@ -546,16 +544,20 @@ for(i=0;i<file_count;i++) {
 # ifdef HAVE_UTIME_H
                            if (stream_flag==FALSE) copyDate(dinfile,outfile);
 # endif
-        	           if (unlink_flag==TRUE && stream_flag==FALSE) remove(dinfile);
+        	           if (sunlink_flag==TRUE && stream_flag==FALSE) x=safe_unlink(dinfile);
+	        	   if (x!=0) perror("safe_unlink");
 #endif        	           
+        	           if (unlink_flag==TRUE && stream_flag==FALSE) unsafe_unlink(dinfile);
+        	           
         	        }
 	        	else {
                           if (stream_flag==FALSE) {
-        	           fprintf (stderr,_("File %s was NOT decrypted successfully.\n"),dinfile);
+                           sprintf(tmperr, _("File %s was NOT decrypted successfully.\n"),dinfile);
+                           err_warn (tmperr);
           	           remove(outfile);
         	          }
         	          else {
-        	           fprintf (stderr,_("Stdin was NOT decrypted successfully.\n"));
+        	           err_warn (_("Stdin was NOT decrypted successfully.\n"));
         	          }
         	        }
         
@@ -588,7 +590,7 @@ for(i=0;i<file_count;i++) {
 	        	 break;
 	        	 
 	        	default:
-                         fprintf(stderr,_("Algorithm unsupported in this version\n"));
+                         err_warn(_("Algorithm unsupported in this version\n"));
 	        	 x=1;
 	        	 break;
                         }
@@ -596,7 +598,7 @@ for(i=0;i<file_count;i++) {
 
                    default:
                     x=1;
-                    fprintf(stderr,_("Mode unsupported in this version\n"));
+                    err_warn (_("Mode unsupported in this version\n"));
                     break;
                   }
 
@@ -611,17 +613,19 @@ for(i=0;i<file_count;i++) {
 # ifdef HAVE_UTIME_H
                            if (stream_flag==FALSE) copyDate(einfile,outfile);
 # endif
-#endif
-	        	   if (unlink_flag==TRUE && stream_flag==FALSE) x=safe_unlink(einfile);
+	        	   if (sunlink_flag==TRUE && stream_flag==FALSE) x=safe_unlink(einfile);
 	        	   if (x!=0) perror("safe_unlink");
+#endif
+	        	   if (unlink_flag==TRUE && stream_flag==FALSE) x=unsafe_unlink(einfile);
 		       	}
 	                else {
                          if (stream_flag==FALSE) {
-	                  fprintf (stderr,_("File %s was NOT encrypted successfully.\n"),einfile);
+                          sprintf(tmperr, _("File %s was NOT encrypted successfully.\n"),einfile);
+                          err_warn (tmperr);
                           remove(outfile);
 	                 }
                          else {
-	                  fprintf (stderr,_("Stdin was NOT encrypted successfully.\n"));
+	                  err_warn (_("Stdin was NOT encrypted successfully.\n"));
 	                 }
 
 	                }
@@ -685,7 +689,10 @@ else {
 
 	}
 	else {
-           if (strlen(key) > 8) {key[8]='\0'; fprintf(stderr,_("Warning: Reducing the size of the key\n"));}
+           if (strlen(key) > 8) {
+             key[8]='\0'; 
+             err_warn(_("Warning: Reducing the size of the key\n"));
+           }
 	   memmove(keyword, key, strlen(key)); /* Copy the 64 bits in key */
 	}
 
@@ -693,7 +700,10 @@ else {
 if (quiet==FALSE) show_mode(ENCRYPT,type,mode,tofile,bit7_flag); 
 
 
-if (unix_setup(keyword)==1) {fprintf(stderr,_("Cannot encrypt key\n"));return 1; }/* init the crypt */
+  if (unix_setup(keyword)==1) {
+    err_warn(_("Cannot encrypt key\n"));
+    return 1; 
+  }/* init the crypt */
 
 do
 {
@@ -844,7 +854,7 @@ if (mode==CBC || mode==CFB) {
   }  
 
    if (td<0) { 
-     fprintf (stderr, _("Unknown algorithm error\n"));
+     err_warn (_("Unknown algorithm error\n"));
      return 1;
    }
 
@@ -1168,9 +1178,9 @@ switch (mode) {
              }
              else { /* eof */
                ch=1;
-               if (i==0) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+               if (i==0) { err_warn(_("Incorrect file type\n")); return 1;}
                memmove(&counter, ciphertext, sizeof(counter));
-               if (counter >= blocksize) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+               if (counter >= blocksize) { err_warn(_("Incorrect file type\n")); return 1;}
              }
            }
 
@@ -1227,9 +1237,9 @@ switch (mode) {
              }
              else { /* eof */
                ch=1;
-               if (i==0) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+               if (i==0) {err_warn(_("Incorrect file type\n")); return 1;}
                memmove(&counter, ciphertext, sizeof(counter));
-               if (counter >= blocksize) {fprintf(stderr,_("Incorrect file type\n")); return 1;}
+               if (counter >= blocksize) {err_warn(_("Incorrect file type\n")); return 1;}
              }
            }
 
@@ -1355,7 +1365,7 @@ switch (mode) {
      return 0; /* crc check passed */
     }
     else {
-     fprintf(stderr, _("CRC32 check failed\n"));
+     err_warn(_("CRC32 check failed\n"));
      return 1; /* CRC32s Do not match */
     }
  }
