@@ -19,11 +19,17 @@
 # include <defines.h>
 #endif
 
+#ifdef HAVE_DEV_RANDOM
+
+static FILE* ffd;
+
 word32 get_rand32() {
  int fd,x;
  word32 rand_data;
 
  fd=open( NAME_OF_DEV_URANDOM, O_RDONLY);
+ if (fd==-1) { perror("/dev/urandom"); exit(-1); }
+
  do {
   x=read( fd, &rand_data, sizeof(rand_data));
  }
@@ -41,6 +47,8 @@ word8 get_rand8() {
  word8 rand_data;
 
  fd=open( NAME_OF_DEV_URANDOM, O_RDONLY);
+ if (fd==-1) { perror("/dev/urandom"); exit(-1); }
+
  do {
   x=read( fd, &rand_data, sizeof(rand_data));
  }
@@ -57,6 +65,8 @@ word32 get_safe_rand32() {
  word32 rand_data;
 
  fd=open( NAME_OF_DEV_RANDOM, O_RDONLY);
+ if (fd==-1) { perror("/dev/urandom"); exit(-1); }
+
  do {
   x=read( fd, &rand_data, sizeof(rand_data));
  }
@@ -74,6 +84,8 @@ word8 get_safe_rand8() {
  word8 rand_data;
 
  fd=open( NAME_OF_DEV_RANDOM, O_RDONLY);
+ if (fd==-1) { perror("/dev/urandom"); exit(-1); }
+
  do {
   x=read( fd, &rand_data, sizeof(rand_data));
  }
@@ -82,3 +94,50 @@ word8 get_safe_rand8() {
 
  return rand_data;
 }
+
+/* When need of many data is required */
+
+int open_rand(int safe) {
+ if (safe==0) {
+   ffd=fopen( NAME_OF_DEV_URANDOM, "rb");
+   if (ffd==NULL) { perror(NAME_OF_DEV_URANDOM); exit(-1); }
+ }
+ else {
+   ffd=fopen( NAME_OF_DEV_RANDOM, "rb");
+   if (ffd==NULL) { perror(NAME_OF_DEV_RANDOM); exit(-1); }
+ }
+ return 0;
+} 
+
+void close_rand() {
+ fclose(ffd);
+}
+
+word32 get_o_rand32() {
+ int x;
+ word32 rand_data;
+
+ do {
+  x=fread( &rand_data, sizeof(rand_data), 1, ffd);
+ }
+ while (x!=sizeof(rand_data));
+
+ return rand_data;
+ 
+}
+
+word8 get_o_rand8() {
+
+ int x;
+ word8 rand_data;
+
+ do {
+  x=fread( &rand_data, sizeof(rand_data),1 ,ffd);
+ }
+ while (x!=sizeof(rand_data));
+
+ return rand_data;
+}
+
+
+#endif
